@@ -1,18 +1,24 @@
 # Common classes
-from .keypoints_utils import *
 from .bbox_utils import *
-from .functional import *
-from .transforms import *
+from .crops.functional import *
+from .crops.transforms import *
 
 # New transformations goes to individual files listed below
 from .domain_adaptation import *
+from .functional import *
 
-from .geometric.transforms import *
 from .geometric.functional import *
 from .geometric.resize import *
 from .geometric.rotate import *
+from .geometric.transforms import *
 
-from .crops.transforms import *
-from .crops.functional import *
+from .dropout.channel_dropout import *
+from .dropout.coarse_dropout import *
+from .dropout.cutout import *
+from .dropout.functional import *
+from .dropout.grid_dropout import *
+from .dropout.mask_dropout import *
 
+from .keypoints_utils import *
+from .transforms import *
 from .utils import *
