@@ -1,17 +1,23 @@
 from __future__ import division
-from collections import defaultdict
-import typing
+
 import random
+import typing
+import warnings
+from collections import defaultdict
 
 import numpy as np
 
-from albumentations.augmentations.keypoints_utils import KeypointsProcessor
-from albumentations.core.serialization import SerializableMeta, get_shortest_class_fullname
-from albumentations.core.six import add_metaclass
-from albumentations.core.transforms_interface import DualTransform, BasicTransform
-from albumentations.core.utils import format_args, Params, get_shape
+from albumentations import random_utils
 from albumentations.augmentations.bbox_utils import BboxProcessor
-from albumentations.core.serialization import SERIALIZABLE_REGISTRY, instantiate_nonserializable
+from albumentations.augmentations.keypoints_utils import KeypointsProcessor
+from albumentations.core.serialization import (
+    SERIALIZABLE_REGISTRY,
+    SerializableMeta,
+    get_shortest_class_fullname,
+    instantiate_nonserializable,
+)
+from albumentations.core.transforms_interface import BasicTransform
+from albumentations.core.utils import Params, format_args, get_shape
 
 __all__ = [
     "BaseCompose",
@@ -27,71 +33,53 @@ __all__ = [
 
 
 REPR_INDENT_STEP = 2
+TransformType = typing.Union[BasicTransform, "BaseCompose"]
+TransformsSeqType = typing.Sequence[TransformType]
 
 
-class Transforms:
-    def __init__(self, transforms):
+def get_always_apply(transforms: typing.Union["BaseCompose", TransformsSeqType]) -> TransformsSeqType:
+    new_transforms: typing.List[TransformType] = []
+    for transform in transforms:  # type: ignore
+        if isinstance(transform, BaseCompose):
+            new_transforms.extend(get_always_apply(transform))
+        elif transform.always_apply:
+            new_transforms.append(transform)
+    return new_transforms
+
+
+class BaseCompose(metaclass=SerializableMeta):
+    def __init__(self, transforms: TransformsSeqType, p: float):
+        if isinstance(transforms, (BaseCompose, BasicTransform)):
+            warnings.warn(
+                "transforms is single transform, but a sequence is expected! Transform will be wrapped into list."
+            )
+            transforms = [transforms]
+
         self.transforms = transforms
-        self.start_end = self._find_dual_start_end(transforms)
-
-    def _find_dual_start_end(self, transforms):
-        dual_start_end = None
-        last_dual = None
-        for idx, transform in enumerate(transforms):
-            if isinstance(transform, DualTransform):
-                last_dual = idx
-                if dual_start_end is None:
-                    dual_start_end = [idx]
-            if isinstance(transform, BaseCompose):
-                inside = self._find_dual_start_end(transform)
-                if inside is not None:
-                    last_dual = idx
-                    if dual_start_end is None:
-                        dual_start_end = [idx]
-        if dual_start_end is not None:
-            dual_start_end.append(last_dual)
-        return dual_start_end
-
-    def get_always_apply(self, transforms):
-        new_transforms = []
-        for transform in transforms:
-            if isinstance(transform, BaseCompose):
-                new_transforms.extend(self.get_always_apply(transform))
-            elif transform.always_apply:
-                new_transforms.append(transform)
-        return Transforms(new_transforms)
-
-    def __getitem__(self, item):
-        return self.transforms[item]
-
-
-def set_always_apply(transforms):
-    for t in transforms:
-        t.always_apply = True
-
-
-@add_metaclass(SerializableMeta)
-class BaseCompose:
-    def __init__(self, transforms, p):
-        self.transforms = Transforms(transforms)
         self.p = p
 
         self.replay_mode = False
         self.applied_in_replay = False
 
-    def __getitem__(self, item):
+    def __len__(self) -> int:
+        return len(self.transforms)
+
+    def __call__(self, *args, **data) -> typing.Dict[str, typing.Any]:
+        raise NotImplementedError
+
+    def __getitem__(self, item: int) -> TransformType:  # type: ignore
         return self.transforms[item]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.indented_repr()
 
-    def indented_repr(self, indent=REPR_INDENT_STEP):
+    def indented_repr(self, indent: int = REPR_INDENT_STEP) -> str:
         args = {k: v for k, v in self._to_dict().items() if not (k.startswith("__") or k == "transforms")}
         repr_string = self.__class__.__name__ + "(["
         for t in self.transforms:
             repr_string += "\n"
             if hasattr(t, "indented_repr"):
-                t_repr = t.indented_repr(indent + REPR_INDENT_STEP)
+                t_repr = t.indented_repr(indent + REPR_INDENT_STEP)  # type: ignore
             else:
                 t_repr = repr(t)
             repr_string += " " * indent + t_repr + ","
@@ -99,21 +87,21 @@ class BaseCompose:
         return repr_string
 
     @classmethod
-    def get_class_fullname(cls):
+    def get_class_fullname(cls) -> str:
         return get_shortest_class_fullname(cls)
 
     @classmethod
-    def is_serializable(cls):
+    def is_serializable(cls) -> bool:
         return True
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         return {
             "__class_fullname__": self.get_class_fullname(),
             "p": self.p,
             "transforms": [t._to_dict() for t in self.transforms],  # skipcq: PYL-W0212
         }
 
-    def get_dict_with_id(self):
+    def get_dict_with_id(self) -> typing.Dict[str, typing.Any]:
         return {
             "__class_fullname__": self.get_class_fullname(),
             "id": id(self),
@@ -121,12 +109,12 @@ class BaseCompose:
             "transforms": [t.get_dict_with_id() for t in self.transforms],
         }
 
-    def add_targets(self, additional_targets):
+    def add_targets(self, additional_targets: typing.Optional[typing.Dict[str, str]]) -> None:
         if additional_targets:
             for t in self.transforms:
                 t.add_targets(additional_targets)
 
-    def set_deterministic(self, flag, save_key="replay"):
+    def set_deterministic(self, flag: bool, save_key: str = "replay") -> None:
         for t in self.transforms:
             t.set_deterministic(flag, save_key)
 
@@ -142,27 +130,34 @@ class Compose(BaseCompose):
         p (float): probability of applying all list of transforms. Default: 1.0.
     """
 
-    def __init__(self, transforms, bbox_params=None, keypoint_params=None, additional_targets=None, p=1.0):
-        super(Compose, self).__init__([t for t in transforms if t is not None], p)
+    def __init__(
+        self,
+        transforms: TransformsSeqType,
+        bbox_params: typing.Optional[typing.Union[dict, "BboxParams"]] = None,
+        keypoint_params: typing.Optional[typing.Union[dict, "KeypointParams"]] = None,
+        additional_targets: typing.Optional[typing.Dict[str, str]] = None,
+        p: float = 1.0,
+    ):
+        super(Compose, self).__init__(transforms, p)
 
-        self.processors = {}
+        self.processors: typing.Dict[str, typing.Union[BboxProcessor, KeypointsProcessor]] = {}
         if bbox_params:
             if isinstance(bbox_params, dict):
-                params = BboxParams(**bbox_params)
+                b_params = BboxParams(**bbox_params)
             elif isinstance(bbox_params, BboxParams):
-                params = bbox_params
+                b_params = bbox_params
             else:
                 raise ValueError("unknown format of bbox_params, please use `dict` or `BboxParams`")
-            self.processors["bboxes"] = BboxProcessor(params, additional_targets)
+            self.processors["bboxes"] = BboxProcessor(b_params, additional_targets)
 
         if keypoint_params:
             if isinstance(keypoint_params, dict):
-                params = KeypointParams(**keypoint_params)
+                k_params = KeypointParams(**keypoint_params)
             elif isinstance(keypoint_params, KeypointParams):
-                params = keypoint_params
+                k_params = keypoint_params
             else:
                 raise ValueError("unknown format of keypoint_params, please use `dict` or `KeypointParams`")
-            self.processors["keypoints"] = KeypointsProcessor(params, additional_targets)
+            self.processors["keypoints"] = KeypointsProcessor(k_params, additional_targets)
 
         if additional_targets is None:
             additional_targets = {}
@@ -175,20 +170,20 @@ class Compose(BaseCompose):
         self.add_targets(additional_targets)
 
         self.is_check_args = True
-        self._disable_check_args_for_transforms(self.transforms.transforms)
+        self._disable_check_args_for_transforms(self.transforms)
 
     @staticmethod
-    def _disable_check_args_for_transforms(transforms: typing.List[typing.Union[BaseCompose, BasicTransform]]):
+    def _disable_check_args_for_transforms(transforms: TransformsSeqType) -> None:
         for transform in transforms:
             if isinstance(transform, BaseCompose):
-                Compose._disable_check_args_for_transforms(transform.transforms.transforms)
+                Compose._disable_check_args_for_transforms(transform.transforms)
             if isinstance(transform, Compose):
                 transform._disable_check_args()
 
-    def _disable_check_args(self):
+    def _disable_check_args(self) -> None:
         self.is_check_args = False
 
-    def __call__(self, *args, force_apply=False, **data):
+    def __call__(self, *args, force_apply: bool = False, **data) -> typing.Dict[str, typing.Any]:
         if args:
             raise KeyError("You have to pass data to augmentations as named arguments, for example: aug(image=image)")
         if self.is_check_args:
@@ -197,7 +192,7 @@ class Compose(BaseCompose):
         need_to_run = force_apply or random.random() < self.p
         for p in self.processors.values():
             p.ensure_data_valid(data)
-        transforms = self.transforms if need_to_run else self.transforms.get_always_apply(self.transforms)
+        transforms = self.transforms if need_to_run else get_always_apply(self.transforms)
 
         check_each_transform = any(
             getattr(item.params, "check_each_transform", False) for item in self.processors.values()
@@ -211,13 +206,14 @@ class Compose(BaseCompose):
 
             if check_each_transform:
                 data = self._check_data_post_transform(data)
+        data = Compose._make_targets_contiguous(data)  # ensure output targets are contiguous
 
         for p in self.processors.values():
             p.postprocess(data)
 
         return data
 
-    def _check_data_post_transform(self, data):
+    def _check_data_post_transform(self, data: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:
         rows, cols = get_shape(data["image"])
 
         for p in self.processors.values():
@@ -228,7 +224,7 @@ class Compose(BaseCompose):
                 data[data_name] = p.filter(data[data_name], rows, cols)
         return data
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         dictionary = super(Compose, self)._to_dict()
         bbox_processor = self.processors.get("bboxes")
         keypoints_processor = self.processors.get("keypoints")
@@ -243,7 +239,7 @@ class Compose(BaseCompose):
         )
         return dictionary
 
-    def get_dict_with_id(self):
+    def get_dict_with_id(self) -> typing.Dict[str, typing.Any]:
         dictionary = super().get_dict_with_id()
         bbox_processor = self.processors.get("bboxes")
         keypoints_processor = self.processors.get("keypoints")
@@ -259,7 +255,7 @@ class Compose(BaseCompose):
         )
         return dictionary
 
-    def _check_args(self, **kwargs):
+    def _check_args(self, **kwargs) -> None:
         checked_single = ["image", "mask"]
         checked_multi = ["masks"]
         check_bbox_param = ["bboxes"]
@@ -276,6 +272,15 @@ class Compose(BaseCompose):
             if internal_data_name in check_bbox_param and self.processors.get("bboxes") is None:
                 raise ValueError("bbox_params must be specified for bbox transformations")
 
+    @staticmethod
+    def _make_targets_contiguous(data: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:
+        result = {}
+        for key, value in data.items():
+            if isinstance(value, np.ndarray):
+                value = np.ascontiguousarray(value)
+            result[key] = value
+        return result
+
 
 class OneOf(BaseCompose):
     """Select one of transforms to apply. Selected transform will be called with `force_apply=True`.
@@ -286,21 +291,21 @@ class OneOf(BaseCompose):
         p (float): probability of applying selected transform. Default: 0.5.
     """
 
-    def __init__(self, transforms, p=0.5):
+    def __init__(self, transforms: TransformsSeqType, p: float = 0.5):
         super(OneOf, self).__init__(transforms, p)
-        transforms_ps = [t.p for t in transforms]
+        transforms_ps = [t.p for t in self.transforms]
         s = sum(transforms_ps)
         self.transforms_ps = [t / s for t in transforms_ps]
 
-    def __call__(self, force_apply=False, **data):
+    def __call__(self, *args, force_apply: bool = False, **data) -> typing.Dict[str, typing.Any]:
         if self.replay_mode:
             for t in self.transforms:
                 data = t(**data)
             return data
 
         if self.transforms_ps and (force_apply or random.random() < self.p):
-            random_state = np.random.RandomState(random.randint(0, 2 ** 32 - 1))
-            t = random_state.choice(self.transforms.transforms, p=self.transforms_ps)
+            idx: int = random_utils.choice(len(self.transforms), p=self.transforms_ps)
+            t = self.transforms[idx]
             data = t(force_apply=True, **data)
         return data
 
@@ -316,30 +321,28 @@ class SomeOf(BaseCompose):
         p (float): probability of applying selected transform. Default: 1.
     """
 
-    def __init__(self, transforms, n, replace=True, p=1):
+    def __init__(self, transforms: TransformsSeqType, n: int, replace: bool = True, p: float = 1):
         super(SomeOf, self).__init__(transforms, p)
         self.n = n
         self.replace = replace
-        transforms_ps = [t.p for t in transforms]
+        transforms_ps = [t.p for t in self.transforms]
         s = sum(transforms_ps)
         self.transforms_ps = [t / s for t in transforms_ps]
 
-    def __call__(self, force_apply=False, **data):
+    def __call__(self, *args, force_apply: bool = False, **data) -> typing.Dict[str, typing.Any]:
         if self.replay_mode:
             for t in self.transforms:
                 data = t(**data)
             return data
 
         if self.transforms_ps and (force_apply or random.random() < self.p):
-            random_state = np.random.RandomState(random.randint(0, 2 ** 32 - 1))
-            transforms = random_state.choice(
-                self.transforms.transforms, size=self.n, replace=self.replace, p=self.transforms_ps
-            )
-            for t in transforms:
+            idx = random_utils.choice(len(self.transforms), size=self.n, replace=self.replace, p=self.transforms_ps)
+            for i in idx:  # type: ignore
+                t = self.transforms[i]
                 data = t(force_apply=True, **data)
         return data
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         dictionary = super(SomeOf, self)._to_dict()
         dictionary.update({"n": self.n, "replace": self.replace})
         return dictionary
@@ -348,12 +351,22 @@ class SomeOf(BaseCompose):
 class OneOrOther(BaseCompose):
     """Select one or another transform to apply. Selected transform will be called with `force_apply=True`."""
 
-    def __init__(self, first=None, second=None, transforms=None, p=0.5):
+    def __init__(
+        self,
+        first: typing.Optional[TransformType] = None,
+        second: typing.Optional[TransformType] = None,
+        transforms: typing.Optional[TransformsSeqType] = None,
+        p: float = 0.5,
+    ):
         if transforms is None:
+            if first is None or second is None:
+                raise ValueError("You must set both first and second or set transforms argument.")
             transforms = [first, second]
         super(OneOrOther, self).__init__(transforms, p)
+        if len(self.transforms) != 2:
+            warnings.warn("Length of transforms is not equal to 2.")
 
-    def __call__(self, force_apply=False, **data):
+    def __call__(self, *args, force_apply: bool = False, **data) -> typing.Dict[str, typing.Any]:
         if self.replay_mode:
             for t in self.transforms:
                 data = t(**data)
@@ -370,16 +383,18 @@ class PerChannel(BaseCompose):
 
     Args:
         transforms (list): list of transformations to compose.
-        channels (list): channels to apply the transform to. Pass None to apply to all.
+        channels (sequence): channels to apply the transform to. Pass None to apply to all.
                          Default: None (apply to all)
         p (float): probability of applying the transform. Default: 0.5.
     """
 
-    def __init__(self, transforms, channels=None, p=0.5):
+    def __init__(
+        self, transforms: TransformsSeqType, channels: typing.Optional[typing.Sequence[int]] = None, p: float = 0.5
+    ):
         super(PerChannel, self).__init__(transforms, p)
         self.channels = channels
 
-    def __call__(self, force_apply=False, **data):
+    def __call__(self, *args, force_apply: bool = False, **data) -> typing.Dict[str, typing.Any]:
         if force_apply or random.random() < self.p:
 
             image = data["image"]
@@ -402,13 +417,19 @@ class PerChannel(BaseCompose):
 
 class ReplayCompose(Compose):
     def __init__(
-        self, transforms, bbox_params=None, keypoint_params=None, additional_targets=None, p=1.0, save_key="replay"
+        self,
+        transforms: TransformsSeqType,
+        bbox_params: typing.Optional[typing.Union[dict, "BboxParams"]] = None,
+        keypoint_params: typing.Optional[typing.Union[dict, "KeypointParams"]] = None,
+        additional_targets: typing.Optional[typing.Dict[str, str]] = None,
+        p: float = 1.0,
+        save_key: str = "replay",
     ):
         super(ReplayCompose, self).__init__(transforms, bbox_params, keypoint_params, additional_targets, p)
         self.set_deterministic(True, save_key=save_key)
         self.save_key = save_key
 
-    def __call__(self, force_apply=False, **kwargs):
+    def __call__(self, *args, force_apply: bool = False, **kwargs) -> typing.Dict[str, typing.Any]:
         kwargs[self.save_key] = defaultdict(dict)
         result = super(ReplayCompose, self).__call__(force_apply=force_apply, **kwargs)
         serialized = self.get_dict_with_id()
@@ -418,30 +439,30 @@ class ReplayCompose(Compose):
         return result
 
     @staticmethod
-    def replay(saved_augmentations, **kwargs):
+    def replay(saved_augmentations: typing.Dict[str, typing.Any], **kwargs) -> typing.Dict[str, typing.Any]:
         augs = ReplayCompose._restore_for_replay(saved_augmentations)
         return augs(force_apply=True, **kwargs)
 
     @staticmethod
-    def _restore_for_replay(transform_dict, lambda_transforms=None):
+    def _restore_for_replay(
+        transform_dict: typing.Dict[str, typing.Any], lambda_transforms: typing.Optional[dict] = None
+    ) -> TransformType:
         """
         Args:
-            transform (dict): A dictionary with serialized transform pipeline.
             lambda_transforms (dict): A dictionary that contains lambda transforms, that
             is instances of the Lambda class.
                 This dictionary is required when you are restoring a pipeline that contains lambda transforms. Keys
                 in that dictionary should be named same as `name` arguments in respective lambda transforms from
                 a serialized pipeline.
         """
-        transform = transform_dict
-        applied = transform["applied"]
-        params = transform["params"]
-        lmbd = instantiate_nonserializable(transform, lambda_transforms)
+        applied = transform_dict["applied"]
+        params = transform_dict["params"]
+        lmbd = instantiate_nonserializable(transform_dict, lambda_transforms)
         if lmbd:
             transform = lmbd
         else:
-            name = transform["__class_fullname__"]
-            args = {k: v for k, v in transform.items() if k not in ["__class_fullname__", "applied", "params"]}
+            name = transform_dict["__class_fullname__"]
+            args = {k: v for k, v in transform_dict.items() if k not in ["__class_fullname__", "applied", "params"]}
             cls = SERIALIZABLE_REGISTRY[name]
             if "transforms" in args:
                 args["transforms"] = [
@@ -455,14 +476,14 @@ class ReplayCompose(Compose):
         transform.applied_in_replay = applied
         return transform
 
-    def fill_with_params(self, serialized, all_params):
+    def fill_with_params(self, serialized: dict, all_params: dict) -> None:
         params = all_params.get(serialized.get("id"))
         serialized["params"] = params
         del serialized["id"]
         for transform in serialized.get("transforms", []):
             self.fill_with_params(transform, all_params)
 
-    def fill_applied(self, serialized):
+    def fill_applied(self, serialized: typing.Dict[str, typing.Any]) -> bool:
         if "transforms" in serialized:
             applied = [self.fill_applied(t) for t in serialized["transforms"]]
             serialized["applied"] = any(applied)
@@ -470,7 +491,7 @@ class ReplayCompose(Compose):
             serialized["applied"] = serialized.get("params") is not None
         return serialized["applied"]
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         dictionary = super(ReplayCompose, self)._to_dict()
         dictionary.update({"save_key": self.save_key})
         return dictionary
@@ -503,13 +524,20 @@ class BboxParams(Params):
             Default: `True`
     """
 
-    def __init__(self, format, label_fields=None, min_area=0.0, min_visibility=0.0, check_each_transform=True):
+    def __init__(
+        self,
+        format: str,
+        label_fields: typing.Optional[typing.Sequence[str]] = None,
+        min_area: float = 0.0,
+        min_visibility: float = 0.0,
+        check_each_transform: bool = True,
+    ):
         super(BboxParams, self).__init__(format, label_fields)
         self.min_area = min_area
         self.min_visibility = min_visibility
         self.check_each_transform = check_each_transform
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         data = super(BboxParams, self)._to_dict()
         data.update(
             {
@@ -545,18 +573,18 @@ class KeypointParams(Params):
 
     def __init__(
         self,
-        format,  # skipcq: PYL-W0622
-        label_fields=None,
-        remove_invisible=True,
-        angle_in_degrees=True,
-        check_each_transform=True,
+        format: str,  # skipcq: PYL-W0622
+        label_fields: typing.Optional[typing.Sequence[str]] = None,
+        remove_invisible: bool = True,
+        angle_in_degrees: bool = True,
+        check_each_transform: bool = True,
     ):
         super(KeypointParams, self).__init__(format, label_fields)
         self.remove_invisible = remove_invisible
         self.angle_in_degrees = angle_in_degrees
         self.check_each_transform = check_each_transform
 
-    def _to_dict(self):
+    def _to_dict(self) -> typing.Dict[str, typing.Any]:
         data = super(KeypointParams, self)._to_dict()
         data.update(
             {
@@ -593,10 +621,10 @@ class Sequential(BaseCompose):
         >>> ])
     """
 
-    def __init__(self, transforms, p=0.5):
+    def __init__(self, transforms: TransformsSeqType, p: float = 0.5):
         super().__init__(transforms, p)
 
-    def __call__(self, **data):
+    def __call__(self, *args, **data) -> typing.Dict[str, typing.Any]:
         for t in self.transforms:
             data = t(**data)
         return data
