@@ -1,10 +1,9 @@
 from __future__ import absolute_import
-from abc import ABCMeta, abstractmethod
 
-from ..core.six import string_types, add_metaclass
+from abc import ABCMeta, abstractmethod
+from typing import Any, Tuple
 
 import numpy as np
-from typing import Any, Tuple
 
 
 def get_shape(img: Any) -> Tuple[int, int]:
@@ -29,14 +28,13 @@ def get_shape(img: Any) -> Tuple[int, int]:
 def format_args(args_dict):
     formatted_args = []
     for k, v in args_dict.items():
-        if isinstance(v, string_types):
+        if isinstance(v, str):
             v = "'{}'".format(v)
         formatted_args.append("{}={}".format(k, v))
     return ", ".join(formatted_args)
 
 
-@add_metaclass(ABCMeta)
-class Params:
+class Params(metaclass=ABCMeta):
     def __init__(self, format, label_fields=None):
         self.format = format
         self.label_fields = label_fields
@@ -45,8 +43,7 @@ class Params:
         return {"format": self.format, "label_fields": self.label_fields}
 
 
-@add_metaclass(ABCMeta)
-class DataProcessor:
+class DataProcessor(metaclass=ABCMeta):
     def __init__(self, params, additional_targets=None):
         self.params = params
         self.data_fields = [self.default_data_name]
