@@ -1,18 +1,11 @@
 import math
 
+import albumentations as A
 import numpy as np
 import pytest
 
 import albumentations.augmentations.geometric.functional as FGeometric
-from albumentations import (
-    HorizontalFlip,
-    VerticalFlip,
-    CenterCrop,
-    RandomSizedCrop,
-    RandomResizedCrop,
-    Compose,
-    NoOp,
-)
+
 from albumentations.augmentations.keypoints_utils import (
     convert_keypoint_from_albumentations,
     convert_keypoints_from_albumentations,
@@ -122,10 +115,10 @@ def test_convert_keypoints_from_albumentations():
 def test_compose_with_keypoint_noop(keypoints, keypoint_format, labels):
     image = np.ones((100, 100, 3))
     if labels is not None:
-        aug = Compose([NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": ["labels"]})
+        aug = A.Compose([A.NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": ["labels"]})
         transformed = aug(image=image, keypoints=keypoints, labels=labels)
     else:
-        aug = Compose([NoOp(p=1.0)], keypoint_params={"format": keypoint_format})
+        aug = A.Compose([A.NoOp(p=1.0)], keypoint_params={"format": keypoint_format})
         transformed = aug(image=image, keypoints=keypoints)
     assert np.array_equal(transformed["image"], image)
     assert transformed["keypoints"] == keypoints
@@ -134,7 +127,7 @@ def test_compose_with_keypoint_noop(keypoints, keypoint_format, labels):
 @pytest.mark.parametrize(["keypoints", "keypoint_format"], [[[[20, 30, 40, 50]], "xyas"]])
 def test_compose_with_keypoint_noop_error_label_fields(keypoints, keypoint_format):
     image = np.ones((100, 100, 3))
-    aug = Compose([NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": "class_id"})
+    aug = A.Compose([A.NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": "class_id"})
     with pytest.raises(Exception):
         aug(image=image, keypoints=keypoints, cls_id=[0])
 
@@ -151,7 +144,7 @@ def test_compose_with_keypoint_noop_error_label_fields(keypoints, keypoint_forma
 )
 def test_compose_with_keypoint_noop_label_outside(keypoints, keypoint_format, labels):
     image = np.ones((100, 100, 3))
-    aug = Compose([NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": list(labels.keys())})
+    aug = A.Compose([A.NoOp(p=1.0)], keypoint_params={"format": keypoint_format, "label_fields": list(labels.keys())})
     transformed = aug(image=image, keypoints=keypoints, **labels)
     assert np.array_equal(transformed["image"], image)
     assert transformed["keypoints"] == keypoints
@@ -162,7 +155,7 @@ def test_compose_with_keypoint_noop_label_outside(keypoints, keypoint_format, la
 def test_random_sized_crop_size():
     image = np.ones((100, 100, 3))
     keypoints = [(0.2, 0.3, 0.6, 0.8), (0.3, 0.4, 0.7, 0.9, 99)]
-    aug = RandomSizedCrop(min_max_height=(70, 90), height=50, width=50, p=1.0)
+    aug = A.RandomSizedCrop(min_max_height=(70, 90), height=50, width=50, p=1.0)
     transformed = aug(image=image, keypoints=keypoints)
     assert transformed["image"].shape == (50, 50, 3)
     assert len(keypoints) == len(transformed["keypoints"])
@@ -171,7 +164,7 @@ def test_random_sized_crop_size():
 def test_random_resized_crop_size():
     image = np.ones((100, 100, 3))
     keypoints = [(0.2, 0.3, 0.6, 0.8), (0.3, 0.4, 0.7, 0.9, 99)]
-    aug = RandomResizedCrop(height=50, width=50, p=1.0)
+    aug = A.RandomResizedCrop(height=50, width=50, p=1.0)
     transformed = aug(image=image, keypoints=keypoints)
     assert transformed["image"].shape == (50, 50, 3)
     assert len(keypoints) == len(transformed["keypoints"])
@@ -180,22 +173,22 @@ def test_random_resized_crop_size():
 @pytest.mark.parametrize(
     ["aug", "keypoints", "expected"],
     [
-        [HorizontalFlip, [[0, 0]], [[2, 0]]],
-        [HorizontalFlip, [[2, 0]], [[0, 0]]],
-        [HorizontalFlip, [[0, 2]], [[2, 2]]],
-        [HorizontalFlip, [[2, 2]], [[0, 2]]],
+        [A.HorizontalFlip, [[0, 0]], [[2, 0]]],
+        [A.HorizontalFlip, [[2, 0]], [[0, 0]]],
+        [A.HorizontalFlip, [[0, 2]], [[2, 2]]],
+        [A.HorizontalFlip, [[2, 2]], [[0, 2]]],
         #
-        [VerticalFlip, [[0, 0]], [[0, 2]]],
-        [VerticalFlip, [[2, 0]], [[2, 2]]],
-        [VerticalFlip, [[0, 2]], [[0, 0]]],
-        [VerticalFlip, [[2, 2]], [[2, 0]]],
+        [A.VerticalFlip, [[0, 0]], [[0, 2]]],
+        [A.VerticalFlip, [[2, 0]], [[2, 2]]],
+        [A.VerticalFlip, [[0, 2]], [[0, 0]]],
+        [A.VerticalFlip, [[2, 2]], [[2, 0]]],
         #
-        [HorizontalFlip, [[1, 1]], [[1, 1]]],
-        [VerticalFlip, [[1, 1]], [[1, 1]]],
+        [A.HorizontalFlip, [[1, 1]], [[1, 1]]],
+        [A.VerticalFlip, [[1, 1]], [[1, 1]]],
     ],
 )
 def test_keypoint_flips_transform_3x3(aug, keypoints, expected):
-    transform = Compose([aug(p=1)], keypoint_params={"format": "xy"})
+    transform = A.Compose([aug(p=1)], keypoint_params={"format": "xy"})
 
     image = np.ones((3, 3, 3))
     transformed = transform(image=image, keypoints=keypoints, labels=np.ones(len(keypoints)))
@@ -205,17 +198,17 @@ def test_keypoint_flips_transform_3x3(aug, keypoints, expected):
 @pytest.mark.parametrize(
     ["aug", "keypoints", "expected"],
     [
-        [HorizontalFlip, [[20, 30, 0, 0]], [[79, 30, 180, 0]]],
-        [HorizontalFlip, [[20, 30, 45, 0]], [[79, 30, 135, 0]]],
-        [HorizontalFlip, [[20, 30, 90, 0]], [[79, 30, 90, 0]]],
+        [A.HorizontalFlip, [[20, 30, 0, 0]], [[79, 30, 180, 0]]],
+        [A.HorizontalFlip, [[20, 30, 45, 0]], [[79, 30, 135, 0]]],
+        [A.HorizontalFlip, [[20, 30, 90, 0]], [[79, 30, 90, 0]]],
         #
-        [VerticalFlip, [[20, 30, 0, 0]], [[20, 69, 0, 0]]],
-        [VerticalFlip, [[20, 30, 45, 0]], [[20, 69, 315, 0]]],
-        [VerticalFlip, [[20, 30, 90, 0]], [[20, 69, 270, 0]]],
+        [A.VerticalFlip, [[20, 30, 0, 0]], [[20, 69, 0, 0]]],
+        [A.VerticalFlip, [[20, 30, 45, 0]], [[20, 69, 315, 0]]],
+        [A.VerticalFlip, [[20, 30, 90, 0]], [[20, 69, 270, 0]]],
     ],
 )
 def test_keypoint_transform_format_xyas(aug, keypoints, expected):
-    transform = Compose(
+    transform = A.Compose(
         [aug(p=1)], keypoint_params={"format": "xyas", "angle_in_degrees": True, "label_fields": ["labels"]}
     )
 
@@ -280,7 +273,7 @@ def test_compose_with_additional_targets():
     image = np.ones((100, 100, 3))
     keypoints = [(10, 10), (50, 50)]
     kp1 = [(15, 15), (55, 55)]
-    aug = Compose([CenterCrop(50, 50)], keypoint_params={"format": "xy"}, additional_targets={"kp1": "keypoints"})
+    aug = A.Compose([A.CenterCrop(50, 50)], keypoint_params={"format": "xy"}, additional_targets={"kp1": "keypoints"})
     transformed = aug(image=image, keypoints=keypoints, kp1=kp1)
     assert transformed["keypoints"] == [(25, 25)]
     assert transformed["kp1"] == [(30, 30)]
@@ -302,3 +295,13 @@ def test_compose_with_additional_targets():
 )
 def test_angle_to_2pi_range(angle, expected):
     assert np.isclose(angle_to_2pi_range(angle), expected)
+
+
+def test_coarse_dropout():
+    aug = A.Compose(
+        [A.CoarseDropout(min_holes=1, max_holes=1, min_height=128, max_width=128, min_width=128, max_height=128, p=1)],
+        keypoint_params=A.KeypointParams(format="xy"),
+    )
+
+    result = aug(image=np.zeros((128, 128)), keypoints=((10, 10), (20, 30)))
+    assert len(result["keypoints"]) == 0
