@@ -4,21 +4,22 @@ import random
 from unittest.mock import patch
 
 import cv2
-import pytest
 import numpy as np
+import pytest
 
 import albumentations as A
 import albumentations.augmentations.functional as F
 from albumentations.core.serialization import SERIALIZABLE_REGISTRY, shorten_class_name
 from albumentations.core.transforms_interface import ImageOnlyTransform
+
 from .conftest import skipif_no_torch
 from .utils import (
     OpenMock,
-    set_seed,
-    get_transforms,
-    get_image_only_transforms,
-    get_dual_transforms,
     check_all_augs_exists,
+    get_dual_transforms,
+    get_image_only_transforms,
+    get_transforms,
+    set_seed,
 )
 
 TEST_SEEDS = (0, 1, 42, 111, 9999)
@@ -312,6 +313,11 @@ AUGMENTATION_CLS_PARAMS = [
     [A.ToSepia, {}],
     [A.Transpose, {}],
     [A.VerticalFlip, {}],
+    [A.RingingOvershoot, dict(blur_limit=(7, 15), cutoff=(np.pi / 5, np.pi / 2))],
+    [A.UnsharpMask, {"blur_limit": 3, "sigma_limit": 0.5, "alpha": 0.2, "threshold": 15}],
+    [A.AdvancedBlur, dict(blur_limit=(3, 5), rotate_limit=(60, 90))],
+    [A.PixelDropout, {"dropout_prob": 0.1, "per_channel": True, "drop_value": None}],
+    [A.PixelDropout, {"dropout_prob": 0.1, "per_channel": False, "drop_value": None, "mask_drop_value": 15}],
 ]
 
 AUGMENTATION_CLS_EXCEPT = {
@@ -677,41 +683,41 @@ def test_lambda_serialization(image, mask, albumentations_bboxes, keypoints, see
 def test_serialization_v2_conversion_without_totensor():
     current_directory = os.path.dirname(os.path.abspath(__file__))
     files_directory = os.path.join(current_directory, "files")
-    transform_0_4_6 = A.load(os.path.join(files_directory, "transform_v0.4.6_without_totensor.json"))
-    with open(os.path.join(files_directory, "output_v0.4.6_without_totensor.json")) as f:
-        output_0_4_6 = json.load(f)
+    transform_1_1_0 = A.load(os.path.join(files_directory, "transform_v1.1.0_without_totensor.json"))
+    with open(os.path.join(files_directory, "output_v1.1.0_without_totensor.json")) as f:
+        output_1_1_0 = json.load(f)
     np.random.seed(42)
     image = np.random.randint(low=0, high=255, size=(256, 256, 3), dtype=np.uint8)
     random.seed(42)
-    transformed_image = transform_0_4_6(image=image)["image"]
-    assert transformed_image.tolist() == output_0_4_6
+    transformed_image = transform_1_1_0(image=image)["image"]
+    assert transformed_image.tolist() == output_1_1_0
 
 
 @skipif_no_torch
 def test_serialization_v2_conversion_with_totensor():
     current_directory = os.path.dirname(os.path.abspath(__file__))
     files_directory = os.path.join(current_directory, "files")
-    transform_0_4_6 = A.load(os.path.join(files_directory, "transform_v0.4.6_with_totensor.json"))
-    with open(os.path.join(files_directory, "output_v0.4.6_with_totensor.json")) as f:
-        output_0_4_6 = json.load(f)
+    transform_1_1_0 = A.load(os.path.join(files_directory, "transform_v1.1.0_with_totensor.json"))
+    with open(os.path.join(files_directory, "output_v1.1.0_with_totensor.json")) as f:
+        output_1_1_0 = json.load(f)
     np.random.seed(42)
     image = np.random.randint(low=0, high=255, size=(256, 256, 3), dtype=np.uint8)
     random.seed(42)
-    transformed_image = transform_0_4_6(image=image)["image"]
-    assert transformed_image.numpy().tolist() == output_0_4_6
+    transformed_image = transform_1_1_0(image=image)["image"]
+    assert transformed_image.numpy().tolist() == output_1_1_0
 
 
 def test_serialization_v2_without_totensor():
     current_directory = os.path.dirname(os.path.abspath(__file__))
     files_directory = os.path.join(current_directory, "files")
     transform = A.load(os.path.join(files_directory, "transform_serialization_v2_without_totensor.json"))
-    with open(os.path.join(files_directory, "output_v0.4.6_without_totensor.json")) as f:
-        output_0_4_6 = json.load(f)
+    with open(os.path.join(files_directory, "output_v1.1.0_without_totensor.json")) as f:
+        output_1_1_0 = json.load(f)
     np.random.seed(42)
     image = np.random.randint(low=0, high=255, size=(256, 256, 3), dtype=np.uint8)
     random.seed(42)
     transformed_image = transform(image=image)["image"]
-    assert transformed_image.tolist() == output_0_4_6
+    assert transformed_image.tolist() == output_1_1_0
 
 
 @skipif_no_torch
@@ -719,13 +725,13 @@ def test_serialization_v2_with_totensor():
     current_directory = os.path.dirname(os.path.abspath(__file__))
     files_directory = os.path.join(current_directory, "files")
     transform = A.load(os.path.join(files_directory, "transform_serialization_v2_with_totensor.json"))
-    with open(os.path.join(files_directory, "output_v0.4.6_with_totensor.json")) as f:
-        output_0_4_6 = json.load(f)
+    with open(os.path.join(files_directory, "output_v1.1.0_with_totensor.json")) as f:
+        output_1_1_0 = json.load(f)
     np.random.seed(42)
     image = np.random.randint(low=0, high=255, size=(256, 256, 3), dtype=np.uint8)
     random.seed(42)
     transformed_image = transform(image=image)["image"]
-    assert transformed_image.numpy().tolist() == output_0_4_6
+    assert transformed_image.numpy().tolist() == output_1_1_0
 
 
 def test_custom_transform_with_overlapping_name():
